<?php
namespace ETC\App\Controllers;

use ETC\App\Controllers\Base_Controller;
use ETC\App\Controllers\Shortcodes\Products as Products_Shortcode;
use ETC\Views\Elementor as View;

/**
 * Elementor initial class.
 *
 * @since      2.0.0
 * @package    ETC
 * @subpackage ETC/Controller
 */
final class Elementor extends Base_Controller {

    /**
     * Minimum Elementor Version Supp
     *
     * @since 2.0.0
     */
    const MINIMUM_ELEMENTOR_VERSION = '2.0.0';

    /**
     * Registered modules.
     *
     * @since 1.0.0
     *
     * @var array
     */
    public static $modules = NULL;

    /**
     * Registered widgets.
     *
     * @since 1.0.0
     *
     * @var array
     */
    public static $widgets = NULL;

    /**
     * Registered controls.
     *
     * @since 1.0.0
     *
     * @var array
     */
    public static $controls = NULL;

    /**
     * Registered google_map_api.
     *
     * @since 1.0.0
     *
     * @var array
     */
    private $google_map_api = NULL;

    /**
     * Constructor
     *
     * @since 2.0.0
     *
     * @access public
     */
    public function __construct() {
        // Register ajax
        // @todo should be move to rest api as best practice
        add_action( 'wp_ajax_etheme_mailchimp' , array( $this, 'mailchimp_widget' ) );
        add_action( 'wp_ajax_nopriv_etheme_mailchimp' , array( $this, 'mailchimp_widget' ) );

        add_action( 'wp_ajax_select2_control', array( $this, '_maybe_post_terms' ) );
        add_action( 'wp_ajax_nopriv_select2_control', array( $this, '_maybe_post_terms' ) );
        
        add_action( 'wp_ajax_et_advanced_tab', array( $this, 'et_advanced_tab' ) );
        add_action( 'wp_ajax_nopriv_et_advanced_tab', array( $this, 'et_advanced_tab' ) );
	
	    add_action( 'wp_ajax_etheme_product_grid_list', array( $this, 'etheme_product_grid_list' ) );
	    add_action( 'wp_ajax_nopriv_etheme_product_grid_list', array( $this, 'etheme_product_grid_list' ) );

        add_action( 'plugins_loaded', array( $this, 'hooks' ) );

    }

    /**
     * Fired elementor options by `plugins_loaded` action hook.
     *
     * @since 2.0.0
     *
     * @access public
     */
    public function hooks() {
        // Check if Elementor installed and activated
        if ( ! did_action( 'elementor/loaded' ) ) {
            return;
        }

        // Check for elementor version
        if ( ! version_compare( ELEMENTOR_VERSION, self::MINIMUM_ELEMENTOR_VERSION, '>=' ) ) {
            add_action( 'admin_notices', array( $this, 'admin_notice_version' ) );
            return;
        }

        $this->register_modules();

        // Register categories, widgets, controls 
        add_action( 'elementor/elements/categories_registered', array( $this, 'register_categories' ) );
        add_action( 'elementor/widgets/widgets_registered', array( $this, 'register_widgets' ) );
        add_action( 'elementor/controls/controls_registered', array( $this, 'register_controls' ) );
        // Elementor editor

		// studio
        add_action( 'init', array( $this, 'enqueue_studio' ) );
	
	    // cross copy paste
	    add_action( 'init', array( $this, 'enqueue_cross_domain_cp' ) );

        add_action( 'elementor/editor/before_enqueue_scripts', array( $this, 'enqueue_editor_styles' ) );
        // Enqueue front end js
        add_action( 'elementor/frontend/after_enqueue_styles', array( $this, 'enqueue_frontend_styles' ) );
        add_action( 'elementor/frontend/after_register_scripts', array( $this, 'enqueue_scripts' ) );

        add_filter( 'elementor/icons_manager/native', array( $this, 'etc_elementor_icons' ) );
        add_filter( 'elementor/fonts/groups', array( $this, 'add_custom_font_group' ) );
        add_filter( 'elementor/fonts/additional_fonts', array( $this, 'add_custom_font' ) );

    }
    
    public function enqueue_studio(){
        if ( get_theme_mod( 'etheme_studio_on', 1) ){
            require_once( ET_CORE_DIR . 'app/models/studio/studio.php' );
        }
    }
	
	/**
	 * Cross Domain Copy Paste.
	 *
	 * @since 1.0.0
	 *
	 * @return void
	 */
	public function enqueue_cross_domain_cp(){
		require_once( ET_CORE_DIR . 'app/models/cross-domain-cp/cross-domain-cp.php' );
	}

    /**  
     * Register widget args  
     *  
     * @return mixed|null|void  
     */  
    public static function module_args() {  
      
        if ( ! is_null( self::$modules ) ) {
            return self::$modules;
        }

        return self::$modules = apply_filters( 'etc/add/elementor/modules', array() );
    }

    /**  
     * Register widget args  
     *  
     * @return mixed|null|void  
     */  
    public static function widgets_args() {  
      
        if ( ! is_null( self::$widgets ) ) {
            return self::$widgets;
        }

        return self::$widgets = apply_filters( 'etc/add/elementor/widgets', array() );
    }

    /**  
     * Register controls args  
     *  
     * @return mixed|null|void  
     */  
    public static function controls_args() {  
      
        if ( ! is_null( self::$controls ) ) {
            return self::$controls;
        }

        return self::$controls = apply_filters( 'etc/add/elementor/controls', array() );
    }

    /**
     * Admin notice when minimum required Elementor version not activating.
     *
     * @since 2.0.0
     *
     * @access public
     */
    public function admin_notice_version() {

        $this->view->elementor_version_requirment(
            array(
                'error_message' => esc_html__( 'Your Elementor version is too old, Please update your elementor plugin to at least '. MINIMUM_ELEMENTOR_VERSION . ' Version', 'xstore-core' ),
            )
        );

    }

    /**
     * Add eight theme Widgets Category
     *
     * @since 2.0.0
     */
    function register_categories( $elements_manager ) {
        $elements_manager->add_category(
            'eight_theme_general',
            array(
                'title' => __( 'XStore Widgets', 'xstore-core' ),
                'icon' => 'fa fa-plug',
            )
        );

    }

    /**
     * Include modules
     *
     * @since 2.0.0
     *
     * @access public
     */
    public function mailchimp_widget() {
        check_ajax_referer( 'etheme_mailchimp', 'security' );

        $return = ['success' => [], 'error' => [] ];
        $dataApi    = function_exists('etheme_get_option') ? etheme_get_option( 'mail_chimp_api', '' ) : '';

        $token      = isset($dataApi) ? $dataApi: '';
        $listed     = isset($_POST['listed']) ? sanitize_key( $_POST['listed'] ): false;
        $email      = isset($_POST['email']) ? sanitize_email( $_POST['email'] ): false;
        $firstname  = isset($_POST['firstname']) ? sanitize_text_field( $_POST['firstname'] ): false;
        $lastname   = isset($_POST['lastname']) ? sanitize_text_field( $_POST['lastname'] ): false;
        $phone      = isset($_POST['phone']) ? sanitize_text_field( $_POST['phone'] ): false;
        
        $data = [
            'email_address' => (($email != '') ? $email : ''),
            'status' => 'subscribed',
            'status_if_new' => 'subscribed',
            'merge_fields' => [
                'FNAME' => (($firstname != '') ? $firstname : ''),
                'LNAME' => (($lastname != '') ? $lastname : ''),
                'PHONE' => (($phone != '') ? $phone : ''),
            ],
        ];

        $server = explode('-', $token);

        if( !is_array($server) || empty($token) || !isset($server[1]) ){
            wp_send_json( array( 'error' => true, 'msg' => esc_html__( 'Please set API Key into Dashboard User Data. ', 'xstore-core' ) ) );
        }
        
        $url = 'https://'.$server[1].'.api.mailchimp.com/3.0/lists/'.$listed.'/members/';

        $response = wp_remote_post( $url, 
            [
                'method' => 'POST',
                'data_format' => 'body',
                'timeout' => 45,
                'headers' => [

                    'Authorization' => 'apikey '.$token,
                    'Content-Type' => 'application/json; charset=utf-8'
                ],
                'body' => json_encode($data )
            ]
        );

        $response = json_decode( wp_remote_retrieve_body( $response ), true );

        if ( isset( $response['status'] ) && 'subscribed' == $response['status'] ) {
            wp_send_json( array( 'error' => false, 'msg' => $response['email_address'] . ' is ' . $response['status'] ) );
        } else {
            wp_send_json( array( 'error' => true, 'msg' => $response['title'] ) );
        }

    }

    /**
     * Ajax select2 control handler
     *
     * @since 2.3.9
     * @return object
     */
    function _maybe_post_terms() {
        
        check_ajax_referer( 'select2_ajax_control', 'security' );

        if ( isset( $_POST['options']['post_type'] ) ) {
            $return = $this->process_post_ajax_select_control();
        }

        wp_send_json( $return );
    }

    /**
     * Get post type for ajax select2 control
     *
     * @since 2.3.9
     * @return return posttype
     */
    function process_post_ajax_select_control() {

        $return = array();

        $args = array( 
            'post_status'           => 'publish', 
            'post_type'             => $_POST['options']['post_type'],
            'ignore_sticky_posts'   => 1,
            'posts_per_page'        => apply_filters('etheme_elementor_post_ajax_results_per_page', 20, $_POST['options']['post_type'])
        );

        // Search
        if ( isset( $_POST['search'] ) && '' != $_POST['search'] ) {
            $args['s'] =  sanitize_text_field( $_POST['search'] );

            $search = $this->_post_get_data_select_control( $args );

            unset( $args['s'] );
        }    

        // Get selected id
        if ( isset( $_POST['id'] ) ) {
            $args['post__in'] =  $_POST['id'];

            $selected = $this->_post_get_data_select_control( $args );
        }

        // Get old options again
        if ( isset( $_POST['old_option'] ) && '' != $_POST['old_option'] ) {
            $args['post__in'] =  $_POST['old_option'] ;

            $old_option = $this->_post_get_data_select_control( $args );
        }

        if ( isset( $selected ) ) {
            return $selected;
        }

        if ( isset( $old_option ) && is_array( $old_option ) && isset( $search ) && is_array( $search ) ) {
            return $old_option + $search;
        } elseif ( isset( $search ) && is_array( $search ) ) {
            return $search;
        } elseif ( isset( $old_option ) && is_array( $old_option ) ) {
            return $old_option;
        }

    }

    protected function _post_get_data_select_control( $args ) {
        $return = array();

        $search_results = new \WP_Query( $args );

        if( $search_results->have_posts() ) :

            while( $search_results->have_posts() ) : $search_results->the_post();   

                $title = ( mb_strlen( $search_results->post->post_title ) > 50 ) ? mb_substr( $search_results->post->post_title, 0, 49 ) . '...' : $search_results->post->post_title;

                $return[$search_results->post->ID] = $title . ' (id - ' . $search_results->post->ID . ')' .
                     (( $search_results->post->post_type == 'product_variation' ) ? ' ' . esc_html__('variation', 'xstore-core') : '' );

            endwhile;

            wp_reset_postdata();

        endif;

        return $return;
    }

    /**
     * Advanced tabs widget
     * 
     * @since 2.3.9
     * @return return tab content
     */
    function et_advanced_tab() {
        // check nonce
    	check_ajax_referer( 'etheme_advancedtabnonce', 'security' );
        // sanitizing
    	$tab_id    = isset( $_POST['tabid'] )    ? sanitize_key( $_POST['tabid'] )    : null;
    	$data_json = isset( $_POST['tabjson'] )  ? $_POST['tabjson']   : null;

        // simple check
    	if ( null === $tab_id ) {
    		wp_send_json_error( array( 'Do not change html via inspect element :)' ) );
    	}

        // Json data
    	if ( is_string( $data_json ) && ! empty( $data_json ) ) {
    		$data_json = json_decode( wp_unslash( $data_json ) , true );
    	}

    	$view = new View;
    	$Products_Shortcode = Products_Shortcode::get_instance();

    	$out = $view->advanced_tabs_ajax(
    		array(
    			'tabs'  				=> $data_json,
    			'Products_Shortcode'  	=> $Products_Shortcode,
    			'is_preview'  			=> ( \Elementor\Plugin::$instance->editor->is_edit_mode() ? true : false ),
    		)
    	);

        echo $out;

        exit();

    }
    
    function etheme_product_grid_list() {
	    check_ajax_referer( 'etheme_product_grid_list_nonce', 'security' );
	    
//	    if( $_POST['context'] !== 'frontend' ) return;
	
//	    global $wpdb, $woocommerce_loop;
//
//	    $attr = array();
//	    $attr = $_POST['attr'];
//	    $output = '';
//	    $args = Array(
//		    'post_type' => 'product',
//		    'post_status' => 'publish',
//		    'ignore_sticky_posts' => 1,
//		    'no_found_rows' => 1,
//		    'posts_per_page' => $attr['posts-per-page'],
//		    'paged' => $attr['paged'],
//		    'orderby' =>'',
//		    'order' => 'ASC',
//	    );
//
     
	    $settings = json_decode( wp_unslash( $_POST['query'] ) , true );
	    $instance = \ETC\App\Controllers\Elementor\General\Product_Grid::get_instance();
	    $products = \ETC\App\Controllers\Elementor\General\Product_Grid::get_query($settings, array(
            'no_found_rows' => 1,
            'paged' => $_POST['attr']['paged'],
            'posts_per_page' => $_POST['attr']['posts-per-page'],
        ));
	    
	    var_dump($instance::get_product_elements());
	    $settings = $instance->get_settings_for_display();
	    var_dump($settings);
//	    wc_set_loop_prop( 'columns', $settings['cols'] );
	    wc_set_loop_prop( 'columns', 4 );
	    wc_set_loop_prop( 'etheme_elementor_product_widget', true );
	    wc_set_loop_prop( 'is_shortcode', true );
	    if ( $products && $products->have_posts() ) {
		
//		    woocommerce_product_loop_start();
		
		    while ( $products->have_posts() ) {
			    $products->the_post();
//			    $this->get_content_product($settings);
			    $instance->get_content_product($instance);
		    }
		
//		    woocommerce_product_loop_end();
	    }
	
	    else {
		    echo '<div class="elementor-panel-alert elementor-panel-alert-warning">'.
		         esc_html__('No products were found matching your selection.', 'xstore-core') .
		         '</div>';
	    }
	
	    wc_reset_loop();
	    wp_reset_postdata();
//	    var_dump($new->get_settings_for_display());
	    die();

//	    $variable_products_detach = etheme_get_option('variable_products_detach', false);
//	    if ( $variable_products_detach ) {
//		    $variable_products_no_parent = etheme_get_option('variation_product_parent_hidden', true);
//		    $args['post_type'][] = 'product_variation';
//		    $posts_not_in = etheme_product_variations_excluded();
//		    if ( array_key_exists('post__not_in', $args) ) {
//			    $args['post__not_in'] = array_unique( array_merge((array)$args['post__not_in'], $posts_not_in) );
//		    }
//		    else {
//			    $args['post__not_in'] = array_unique( $posts_not_in );
//		    }
//		    // hides all variable products
//		    if ( $variable_products_no_parent ) {
//			    $args['tax_query'][] = array(
//				    array(
//					    'taxonomy' => 'product_type',
//					    'field'    => 'slug',
//					    'terms'    => 'variable',
//					    'operator' => 'NOT IN',
//				    ),
//			    );
//		    }
//	    }
//
//	    if ( $attr['orderby'] ) {
//		    $args['orderby'] = $attr['orderby'];
//	    } else {
//		    $args['orderby'] = '';
//	    }
//
//	    if ( $attr['order'] ) {
//		    $args['order'] = $attr['order'];
//	    } else {
//		    $args['order'] = 'ASC';
//	    }
//
//	    if ( isset( $attr['stock'] ) ) {
//		    $args['meta_query'] = array(
//			    array (
//				    'key' => '_stock_status',
//				    'value' => 'instock',
//				    'compare' => '='
//			    ),
//		    );
//	    }
//
//	    $args['tax_query'][] = array(
//		    'taxonomy' => 'product_visibility',
//		    'field'    => 'name',
//		    'terms'    => 'hidden',
//		    'operator' => 'NOT IN',
//	    );
//
//	    if (  isset( $attr['type'] ) ) {
//		    switch ($attr['type']) {
//			    case 'featured':
//				    $args['tax_query'][] = array(
//					    'taxonomy' => 'product_visibility',
//					    'field'    => 'name',
//					    'terms'    => 'featured',
//					    'operator' => 'IN',
//				    );
//				    break;
//			    case 'bestsellings':
//				    $args['meta_key'] = 'total_sales';
//				    $args['orderby'] = 'meta_value_num';
//				    break;
//			    default:
//
//				    break;
//		    }
//	    }
//
//	    if ($attr['orderby'] == 'price') {
//		    $args['meta_key'] = '_price';
//		    $args['orderby'] = 'meta_value_num';
//	    }
//
//	    if ( isset($attr['ids']) ) {
//		    $ids = explode( ',', $attr['ids'] );
//		    $ids = array_map('trim', $ids);
//		    $args['post__in'] = $ids;
//	    }
//
//	    // Narrow by categories
//	    if( ! empty( $attr['taxonomies'] ) ) {
//		    $taxonomy_names = get_object_taxonomies( 'product' );
//		    $terms = get_terms( $taxonomy_names, array(
//			    'orderby' => 'name',
//			    'include' => $attr['taxonomies']
//		    ));
//
//		    if( ! is_wp_error( $terms ) && ! empty( $terms ) ) {
//			    $args['tax_query'] = array('relation' => 'OR');
//			    foreach ($terms as $key => $term) {
//				    $args['tax_query'][] = array(
//					    'taxonomy' => $term->taxonomy,
//					    'field' => 'slug',
//					    'terms' => array( $term->slug ),
//					    'include_children' => true,
//					    'operator' => 'IN'
//				    );
//			    }
//		    }
//	    }
//
//	    $products = new WP_Query( $args );
//	    $class = '';
//
//	    wc_setup_loop( array(
//		    'columns'      => $attr['columns'],
//		    'name'         => 'product',
//		    'is_shortcode' => true,
//		    'total'        => $args['posts_per_page'],
//	    ) );
//
//	    if ( isset( $attr['limit'] ) ) {
//		    $_i = 0;
//	    }
//
////	    if ( isset($attr['product_view']) )
////		    $woocommerce_loop['product_view'] = $attr['product_view'];
////	    if ( isset($attr['custom_template']) )
////		    $woocommerce_loop['custom_template'] = $attr['custom_template'];
////	    if ( isset($attr['product_view_color']) )
////		    $woocommerce_loop['product_view_color'] = $attr['product_view_color'];
////	    if ( isset($attr['hover']) )
////		    $woocommerce_loop['hover'] = $attr['hover'];
////	    if ( isset($attr['size']) )
////		    $woocommerce_loop['size'] = $attr['size'];
////	    if ( isset($attr['show_counter']) )
////		    $woocommerce_loop['show_counter'] = $attr['show_counter'];
////	    if ( isset($attr['show_stock']) )
////		    $woocommerce_loop['show_stock'] = $attr['show_stock'];
////
////	    if ( isset($attr['product_content_elements']) ) {
////		    $product_content_elements = explode(',', $attr['product_content_elements']);
////		    $woocommerce_loop['product_content_elements'] = $product_content_elements;
////		    if ( in_array('product_page_product_excerpt', $product_content_elements) ) {
////			    $woocommerce_loop['show_excerpt']       = true;
////		    }
////		    if ( isset($attr['product_add_to_cart_quantity']))
////			    $woocommerce_loop['product_add_to_cart_quantity'] = $attr['product_add_to_cart_quantity'];
////	    }
////
////	    $woocommerce_loop['loading_class'] = 'productAnimated product-fade';
//
//	    if ( $products->have_posts() ) :
//		    if ( wc_get_loop_prop( 'total' ) ) {
//
//			    ?>
<!--			    --><?php //woocommerce_product_loop_start(false); ?>
<!--			-->
<!--			    --><?php //while ( $products->have_posts() ) : $products->the_post(); ?>
<!--				-->
<!--				    --><?php
//				    if ( isset( $attr['limit'] ) ) {
//					    if ( $_i >= $attr['limit'] ) {
//						    break;
//					    }
//					    $_i++;
//				    }
//				    ?>
<!--				-->
<!--				    --><?php //// $output .= wc_get_template_part( 'content', 'product' );
//				    $output .= 'Product '.$_i;
//				    ?>
<!--			-->
<!--			    --><?php //endwhile; // end of the loop. ?>
<!--			-->
<!--			    --><?php //woocommerce_product_loop_end(false); ?>
<!--		    --><?php //} ?>
<!--	    --><?php //endif;
//
//	    if ( isset($woocommerce_loop['product_view']) )
//		    unset($woocommerce_loop['product_view']);
//	    if ( isset($woocommerce_loop['custom_template']) )
//		    unset($woocommerce_loop['custom_template']);
//	    if ( isset($woocommerce_loop['product_view_color']) )
//		    unset($woocommerce_loop['product_view_color']);
//	    if ( isset($woocommerce_loop['hover']) )
//		    unset($woocommerce_loop['hover']);
//	    if ( isset($woocommerce_loop['size']) )
//		    unset($woocommerce_loop['size']);
//	    if ( isset($woocommerce_loop['show_counter']) )
//		    unset($woocommerce_loop['show_counter']);
//	    if ( isset($woocommerce_loop['show_stock']) )
//		    unset($woocommerce_loop['show_stock']);
//
//	    if ( isset($woocommerce_loop['product_content_elements']))
//		    unset($woocommerce_loop['product_content_elements']);
//
//	    if ( isset($woocommerce_loop['product_add_to_cart_quantity']))
//		    unset($woocommerce_loop['product_add_to_cart_quantity']);
//
//	    if ( isset($woocommerce_loop['show_excerpt']))
//		    unset($woocommerce_loop['show_excerpt']);
//
//	    unset($woocommerce_loop['loading_class']);
//
//	    wp_reset_postdata();
//	    wc_reset_loop();
//
//	    echo $output; // All data escaped
//
//	    die();
    }

    /**
     * Include modules
     *
     * @since 2.0.0
     *
     * @access public
     */
    public function register_modules() {

        $modules = self::module_args();
        foreach ( $modules as $module ) {
            foreach ( $module as $class ) {
                new $class();
            }
        }

    }

    /**
     * Include widgets files and register them
     *
     * @since 2.0.0
     *
     * @access public
     */
    public function register_widgets( $widgets_manager ) {

        $args = self::widgets_args();
        foreach ( $args as $widget_classes ) {
            foreach ( $widget_classes as $class ) {
                $widgets_manager->register_widget_type( new $class() );
            }
        }

    }

    /**
     * Register controls
     *
     * @since 1.0.0
     *
     * @access public
     */
    public function register_controls( $controls_manager ) {

        $args = self::controls_args();
        foreach ( $args as $control_type => $control ) {
            $controls_manager->register_control( $control_type, new $control['class']() );
        }
    }

    public function etc_elementor_icons( $icons_library ) {

        $icons_library['xstore-icons'] = [
            'name' => 'xstore-icons',
            'label' => __( 'XStore Icons', 'xstore-core' ),
            'url' => self::get_et_asset_url( ( get_theme_mod('bold_icons', 0) ? 'bold' : 'light') ),
            'enqueue' => [ self::get_et_asset_url( 'xstore-icons' ) ],
            'prefix' => 'et-',
            'displayPrefix' => 'et-icon',
            'labelIcon' => 'et-icon et-star-o',
            'ver' => '1.3.0',
            'fetchJson' => self::get_et_asset_url( 'light', 'js', false ),
            'native' => true,
        ];

        return $icons_library;
    }

    private static function get_et_asset_url( $filename, $ext_type = 'css', $add_suffix = true ) {
        // static $is_test_mode = null;
        // if ( null === $is_test_mode ) {
        //     $is_test_mode = defined( 'SCRIPT_DEBUG' ) && SCRIPT_DEBUG || defined( 'ELEMENTOR_TESTS' ) && ELEMENTOR_TESTS;
        // }
        $url = ET_CORE_URL . 'app/assets/lib/xstore-icons/' . $ext_type . '/' . $filename;
        // if ( ! $is_test_mode && $add_suffix ) {
        //     $url .= '.min';
        // }
        return $url . '.' . $ext_type;
    }

    /**
     * Add xstore custom fonts group
     * 
     * @since    2.5.0
     * @param new fonts
     */
    public function add_custom_font_group( $font_groups ) {
        // Get available fonts
        $uploaded_fonts = get_option( 'etheme-fonts', false );

        if ( ! $uploaded_fonts ) {
            return $font_groups;
        }

        $new_group =  array(
            'xstore' => 'XStore Fonts'
        );

        return array_merge( $font_groups, $new_group );
    }

    /**
     * add xstore custom font
     *
     * @since    2.5.0
     */
    public function add_custom_font( $additional_fonts ) {
        $uploaded_fonts = get_option( 'etheme-fonts', false );

        if ( false == $uploaded_fonts || is_null( $uploaded_fonts ) ) {
            return $additional_fonts;
        }

        $new_fonts = array();

        foreach ( $uploaded_fonts as $font ) {
            $new_fonts[$font['name']] = 'xstore';
        }

        return array_merge( $additional_fonts ,$new_fonts );
    }

    /**
     * Register the stylesheets for elementor.
     *
     * @since    2.0.0
     */
    public function enqueue_editor_styles() {

        wp_enqueue_style(
            'et-core-elementor-style',
            ET_CORE_URL . 'app/assets/css/elementor-editor.css',
            array(),
            ET_CORE_VERSION,
            'all'
        );

        wp_enqueue_style(
            'et-core-eight_theme-elementor-icon',
            ET_CORE_URL . 'app/assets/css/eight_theme-elementor-icon.css',
            array(),
            ET_CORE_VERSION,
            'all'
        );

        if ( get_theme_mod( 'google_map_api', '' ) ) {
            $this->google_map_api = get_theme_mod( 'google_map_api', '' );
        }

        if( $this->google_map_api != '' ) {
            $url = 'https://maps.googleapis.com/maps/api/js?key='. $this->google_map_api .'&language='.get_locale();
        } else {
            $url = 'https://maps.googleapis.com/maps/api/js?language='.get_locale();
        }

        wp_enqueue_script( 
            'etheme-google-map-admin-api', 
            $url, 
            ['elementor-editor'], 
            ET_CORE_VERSION, 
            true  
        );

        wp_enqueue_script( 
            'etheme-google-map-admin', 
            ET_CORE_URL . 'app/assets/js/google-map-admin.js', 
            array( 'etheme-google-map-admin-api' ), 
            ET_CORE_VERSION, 
            true 
        );

        wp_enqueue_script( 
            'et-elementor-editor', 
            ET_CORE_URL . 'app/assets/js/editor-before.js', 
            array(),
            ET_CORE_VERSION 
        );

        // icons 
        // wp_enqueue_script(
        //     'font-awesome-4-shim',
        //     self::get_fa_asset_url( 'v4-shims', 'js' ),
        //     [],
        //     ELEMENTOR_VERSION
        // );
        // wp_enqueue_style(
        //     'font-awesome-5-all',
        //     self::get_fa_asset_url( 'all' ),
        //     [],
        //     ELEMENTOR_VERSION
        // );
        // wp_enqueue_style(
        //     'font-awesome-4-shim',
        //     self::get_fa_asset_url( 'v4-shims' ),
        //     [],
        //     ELEMENTOR_VERSION
        // );

    }

    /**
     * Register the stylesheets for elementor.
     *
     * @since    2.0.0
     */
    public function enqueue_frontend_styles() {
    	if (\Elementor\Plugin::$instance->editor->is_edit_mode() || \Elementor\Plugin::$instance->preview->is_preview_mode()){
		    wp_enqueue_style( 'et-core-elementor-editor-style', ET_CORE_URL . 'app/assets/css/elementor-editor-preview.css', ['editor-preview'], '1.0' );
	    }
    }

    /**
     * Register the JavaScript for elementor.
     *
     * @since    2.0.0
     */
    public function enqueue_scripts() {
    	
    	$locate_lang = get_locale();

    	// @todo empty file to remove but make correct localize for all scripts depends on it
        wp_enqueue_script(
            'et-core-elementor-script',
            ET_CORE_URL . 'app/assets/js/elementor.js',
            array( 'jquery' ),
            ET_CORE_VERSION,
            false
        );

        // wp_enqueue_script( 
        //     'etheme-mail-chimp', 
        //     ET_CORE_URL . 'app/assets/js/mail-chimp.js', 
        //     array('jquery'), 
        //     ET_CORE_VERSION, 
        //     false 
        // );
	    
	    $localize = array(
		    'adminajax'     => admin_url( 'admin-ajax.php' ),
		    'search' => array(
			    'noResults'               => esc_html__( 'No results were found!', 'xstore-core' ),
			    'product' 				 => esc_html__('Products', 'xstore-core'),
			    'page' 				 => esc_html__('Pages', 'xstore-core'),
			    'post' 					 => esc_html__('Posts', 'xstore-core'),
			    'etheme_portfolio' 			 => esc_html__('Portfolio', 'xstore-core'),
			    'product_found' => esc_html__('{{count}} Products found', 'xstore-core'),
			    'page_found' => esc_html__('{{count}} Pages found', 'xstore-core'),
			    'post_found' => esc_html__('{{count}} Posts found', 'xstore-core'),
			    'etheme_portfolio_found' => esc_html__('{{count}} Portfolio found', 'xstore-core'),
			    'custom_post_type_found' => esc_html__('{{count}} {{post_type}} found', 'xstore-core'),
			    'show_more' => esc_html__('Show {{count}} more', 'xstore-core'),
			    'show_all' => esc_html__('View all results', 'xstore-core'),
			    'items_found' => esc_html__('{{count}} items found', 'xstore-core'),
			    'item_found' => esc_html__('{{count}} item found', 'xstore-core'),
//			    'single_product_builder' => get_query_var( 'etheme_single_product_builder', false ),
//			    'fancy_select_categories' => get_theme_mod('search_category_fancy_select_et-desktop', false), //
			   
		    )
	    );
	    $localize['search']['noSuggestionNoticeWithMatches'] = $localize['search']['noResults'] . '<p>' . __( 'No items matched your search {{search_value}}.', 'xstore-core' ) . '</p>';
	    $localize['lottie']['defaultAnimationUrl'] = ET_CORE_URL . 'app/assets/js/lottie-default.json';

        wp_localize_script(
            'et-core-elementor-script',
            'etheme_elementor_localize',
            $localize
        );

        if ( get_theme_mod( 'google_map_api', '' ) ) {
            $this->google_map_api = get_theme_mod( 'google_map_api', '' );
        }


        if( $this->google_map_api != '' ) {
            $url = 'https://maps.googleapis.com/maps/api/js?key='. $this->google_map_api .'&language='.$locate_lang;
        } else {
            $url = 'https://maps.googleapis.com/maps/api/js?language='.$locate_lang;
        }

        wp_register_script( 
            'etheme-google-map-api', 
            $url, 
            array(), 
            ET_CORE_VERSION, 
            true  
        );

        wp_localize_script( 
            'etheme-google-map-api', 
            'etheme_google_map_loc', 
            array( 'plugin_url' => ET_CORE_URL )
        );

        wp_register_script( 
            'etheme-google-map', 
            ET_CORE_URL . 'app/assets/js/google-map.js', 
            array( 'etheme-google-map-api' ), 
            ET_CORE_VERSION, 
            true 
        );
	
	    wp_localize_script(
		    'etheme_facebook_sdk',
		    'etheme_facebook_sdk_config',
		    array(
		    	'facebook_sdk' => array(
				    'lang' => $locate_lang,
				    'app_id' => get_option( 'etheme_facebook_app_id', get_theme_mod('facebook_app_id', '') ),
			    )
		    )
	    );
    }

}